from __future__ import absolute_import

import itertools


class DirectoryWalker(object):
    def __init__(self, writer):
        """
        :type writer: ncdu_s3.NcduDataWriter
        """

        self.writer = writer
        self.current_path = []

    def process_item(self, path, size):
        key_filename = path.pop()

        if self.current_path != path:
            # update our position in the directory hierarchy
            conflict = False
            add_dirs = []

            for p1, p2 in itertools.izip_longest(self.current_path, path):
                if p1 != p2:
                    # first conflict starts another logic in our code
                    conflict = True

                if conflict:
                    if p1 is not None:
                        self.writer.dir_leave()

                    if p2 is not None:
                        add_dirs.append(p2)

            for d in add_dirs:
                # ncdu doesn't support empty dir names. Replace '' with '<empty>'
                self.writer.dir_enter(d if d != '' else '<empty>')

            self.current_path = path

        # directory entry ends with a '/' so the key_filename will be ''.
        # in that case, omit it
        if key_filename != '':
            self.writer.file_entry(key_filename, size)



from __future__ import absolute_import
import boto3
import click
import itertools
import urlparse
from ncdu_s3 import NcduDataWriter, DirectoryWalker, S3DirectoryGenerator


@click.command()
@click.argument('s3-url')
@click.argument('output', type=click.File('w'))
@click.pass_context
def main(ctx, s3_url, output):
    assert isinstance(ctx, click.Context)

    try:
        s3_directory_generator = S3DirectoryGenerator(boto3.resource('s3'), s3_url)
    except SyntaxError, e:
        ctx.fail(e.message)
        return

    with NcduDataWriter(output, s3_url) as ncdu:
        walker = DirectoryWalker(ncdu)

        for path, size in s3_directory_generator:
            walker.process_item(path, size)

if __name__ == '__main__':
    main()



from __future__ import absolute_import

import time
import ujson as json


class NcduDataWriter(object):
    """
    Write ncdu formatted data files (like ncdu -o)

    I couldn't find any good streaming JSON writers for python, so this is a bit hackish, but safe.
    All JSON objects are written with json.dump(), the only stuff I write directly to the file are
    ints, newlines, commas and square brackets
    """

    def __init__(self, output, root):
        """
        :type output: io.RawIOBase
        :type root: str
        """

        self.output = output
        self.depth = 0

        self.output.write('[1,0,')
        json.dump({'progname': 'ncdu-s3', 'progver': '0.1', 'timestamp': int(time.time())}, self.output)

        # ncdu data format must begin with a directory
        self.dir_enter(root)

    def __enter__(self):
        return self

    def __exit__(self, *_):
        self.close()

    def dir_enter(self, name):
        """
        :type name: str
        """

        self.depth += 1

        self.output.write(",\n")

        self.output.write('[')
        json.dump({'name': name}, self.output)

    def dir_leave(self):
        if self.depth > 0:
            self.depth -= 1
            self.output.write(']')

    def file_entry(self, name, size):
        """
        :type name: str
        :type size: int
        """

        self.output.write(",\n")

        json.dump({'name': name, 'dsize': size}, self.output)

    def close(self):
        for i in xrange(self.depth):
            self.dir_leave()

        # close the format JSON document we opened in our constructor
        self.output.write(']')


from __future__ import absolute_import

import urlparse


class S3DirectoryGenerator(object):
    def __init__(self, s3_connection, s3_url):
        parsed_s3_url = urlparse.urlparse(s3_url)
        if parsed_s3_url.scheme != 's3':
            raise SyntaxError('Invalid S3 scheme')

        self.bucket_name = parsed_s3_url.netloc
        self.bucket_path = parsed_s3_url.path[1:] if parsed_s3_url.path.startswith('/') else parsed_s3_url.path
        bucket_path_split = self.bucket_path.split('/')

        self.bucket = s3_connection.Bucket(self.bucket_name)

        if bucket_path_split[-1] == '':
            # directory listing
            self.strip_length = len(self.bucket_path)
        else:
            # prefix listing
            self.strip_length = len('/'.join(bucket_path_split[:-1]))

    def __iter__(self):
        return self.generator()

    def generator(self):
        for o in self.bucket.objects.filter(Prefix=self.bucket_path):
            key = o.key[self.strip_length:]

            # S3 doesn't really have a concept of dirs. The convention is '/' is path separator, we do the same
            path = key.split('/')

            if path[0] == '' and not self.bucket_path.endswith('/'):
                # we assume the S3 prefix is a directory that wasn't terminated with a '/'
                path.pop(0)

            yield (path, o.size)



from setuptools import setup, find_packages

setup(
    name='ncdu-s3',
    version='0.2',
    description='Generate ncdu data files for S3 buckets',
    author='EverythingMe',
    author_email='omrib@everything.me',
    url='http://github.com/EverythingMe/ncdu-s3',
    packages=find_packages(),
    install_requires=['boto3', 'click', 'ujson'],

    entry_points={
        'console_scripts': [
            'ncdu-s3 = ncdu_s3.main:main'
        ]
    },

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: System :: Systems Administration',
        'Topic :: Utilities'
    ]
)

